@@ -1,42 +1,34 @@
+from __future__ import unicode_literals
+
 import django
+from distutils.version import LooseVersion
 from django.contrib import admin, messages
 from django.contrib.admin import helpers
-from django.contrib.admin.models import LogEntry, CHANGE
-try:
-    # Django 1.7
-    from django.contrib.admin.utils import model_ngettext
-except ImportError:
-    # Django < 1.7
-    from django.contrib.admin.util import model_ngettext
+from django.contrib.admin.models import CHANGE, LogEntry
+from django.contrib.admin.utils import model_ngettext
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import PermissionDenied
-
-try:
-    # Django >= 1.3
-    from django.template.response import TemplateResponse
-except ImportError:
-    from django.shortcuts import render_to_response
-    from django.template import RequestContext
-
-try:
-    # Django > 1.3
-    from django.utils.encoding import force_text
-except ImportError:
-    # Django 1.3
-    from django.utils.encoding import force_unicode as force_text
+from django.template.response import TemplateResponse
+from django.utils.encoding import force_text
+from django.utils.html import conditional_escape, format_html
+from django.utils.six import text_type
 from django.utils.translation import ugettext_lazy as _
 
+from .utils import related_objects
+
 
 def highlight_deleted(obj):
     """
         Display in red lines when object is deleted.
     """
+    obj_str = conditional_escape(text_type(obj))
     if not getattr(obj, 'deleted', False):
-        return obj
+        return obj_str
     else:
-        return '<span class="deleted">{0}</span>'.format(obj)
+        return format_html('<span class="deleted">{0}</span>', obj_str)
+
+
 highlight_deleted.short_description = _("Name")
-highlight_deleted.allow_tags = True
 
 
 class SafeDeleteAdmin(admin.ModelAdmin):
@@ -48,7 +40,7 @@ class SafeDeleteAdmin(admin.ModelAdmin):
         >>> from safedelete.admin import SafeDeleteAdmin, highlight_deleted
         >>> class ContactAdmin(SafeDeleteAdmin):
         ...    list_display = (highlight_deleted, "first_name", "last_name", "email") + SafeDeleteAdmin.list_display
-        ...    list_filter = ("last_name") + SafeDeleteAdmin.list_filter
+        ...    list_filter = ("last_name",) + SafeDeleteAdmin.list_filter
     """
     undelete_selected_confirmation_template = "safedelete/undelete_selected_confirmation.html"
 
@@ -71,18 +63,16 @@ class SafeDeleteAdmin(admin.ModelAdmin):
 
     def get_queryset(self, request):
         try:
-            qs = self.model._default_manager.all_with_deleted()
+            queryset = self.model.all_objects.all()
         except:
-            qs = self.model._default_manager.all()
+            queryset = self.model._default_manager.all()
 
-        if hasattr(self, 'get_ordering'):
-            # Django >= 1.4
-            ordering = self.get_ordering(request)
-            if ordering:
-                qs = qs.order_by(*ordering)
-        return qs
+        ordering = self.get_ordering(request)
+        if ordering:
+            queryset = queryset.order_by(*ordering)
+        return queryset
 
-    def log_undeletion(self, request, object, object_repr):
+    def log_undeletion(self, request, obj, object_repr):
         """
         Log that an object will be undeleted.
 
@@ -91,7 +81,7 @@ class SafeDeleteAdmin(admin.ModelAdmin):
         LogEntry.objects.log_action(
             user_id=request.user.pk,
             content_type_id=ContentType.objects.get_for_model(self.model).pk,
-            object_id=object.pk,
+            object_id=obj.pk,
             object_repr=object_repr,
             action_flag=CHANGE
         )
@@ -102,6 +92,8 @@ class SafeDeleteAdmin(admin.ModelAdmin):
             raise PermissionDenied
         assert hasattr(queryset, 'undelete')
 
+        # Remove not deleted item from queryset
+        queryset = queryset.filter(deleted__isnull=False)
         # Undeletion confirmed
         if request.POST.get('post'):
             n = queryset.count()
@@ -135,6 +127,8 @@ class SafeDeleteAdmin(admin.ModelAdmin):
             objects_name = force_text(opts.verbose_name_plural)
         title = _("Are you sure?")
 
+        related_list = [list(related_objects(obj)) for obj in queryset]
+
         context = {
             'title': title,
             'objects_name': objects_name,
@@ -142,19 +136,20 @@ class SafeDeleteAdmin(admin.ModelAdmin):
             "opts": opts,
             "app_label": opts.app_label,
             'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
+            'related_list': related_list
         }
 
-        if django.VERSION[1] < 3:
-            return render_to_response(
-                "safedelete/undelete_selected_confirmation_django12.html",
-                context,
-                context_instance=RequestContext(request))
-        else:
+        if LooseVersion(django.get_version()) < LooseVersion('1.10'):
             return TemplateResponse(
                 request,
                 self.undelete_selected_confirmation_template,
                 context,
                 current_app=self.admin_site.name,
             )
-
+        else:
+            return TemplateResponse(
+                request,
+                self.undelete_selected_confirmation_template,
+                context,
+            )
     undelete_selected.short_description = _("Undelete selected %(verbose_name_plural)s.")
